Option Strict Off
Option Explicit On
Module modErrorHandler
	'---------------------------------------------------------------
	' Module Name: ErrorHandler.Bas
	'
	' Function:    General error handling procedures
	'
	' References:  None... this is a building brick for any application
	'
    ' Created By:  Simon Dowdell / Andrew Johnston
    '---------------------------------------------------------------
	
	Public Const isgUserDefinedError As Short = 96 'User defined error
    Public Const isgSevereUserDefinedError As Single = vbObjectError + 512 + 2 'Severe error
	Public Const isgAssertionError As Single = vbObjectError + 512 + 3 'Assertion error
	Public Const isgOracleError As Single = vbObjectError + 512 + 4 'User defined Oracle error
    '***************************************************************************************
	' Sub:    RaiseUserDefinedError
	' --------------------------------------------------------------------------------------
	' Purpose:      Causes VB to raise a user defined errpr
	' Parameters:   Description - A description of the error
	'               ErrorGroup - OPTIONAL... either isgUserDefinedError or isgSeverUserDefinedError
	'***************************************************************************************
	Public Sub RaiseUserDefinedError(ByRef ErrorDescription As String, Optional ByRef ErrorGroup As Integer = 0)
		Err.Description = ErrorDescription 'Set the error description
        Select Case True
            Case IsNothing(ErrorGroup)
                Err.Number = isgUserDefinedError 'Default error number to isgUserDefinedError
            Case ErrorGroup = isgSevereUserDefinedError, ErrorGroup = isgUserDefinedError
                Err.Number = ErrorGroup 'Set error number to valid code
            Case Else
                Err.Number = isgUserDefinedError 'Default error number to isgUserDefinedError
        End Select
        Err.Raise(Err.Number) 'Re-raise the error
	End Sub
	'***************************************************************************************
	' Sub:    ClassErrorHandler
	' --------------------------------------------------------------------------------------
	' Purpose:      Handles untrapped errors generated in class modules
	' Parameters:   ModuleName - The module name where the error occurred
	'               ProcedureName - The procedure name where the error occurred
	'               ExtraDebugInfo - Any additional debuging information
	'***************************************************************************************
	Public Sub ClassErrorHandler(ByRef ModuleName As String, ByRef ProcedureName As String, Optional ByRef ExtraDebugInfo As String = "")
		
        Err.Raise(Err.Number) 'Re-raise the error
    End Sub
#If UserServices Then
    '***************************************************************************************
    ' Sub:    FormErrorHandler
    ' --------------------------------------------------------------------------------------
    ' Purpose:      Handles untrapped errors generated in form modules
    '               Any errors will be displayed on the errors form
    '               NB: only compiled into the exe when UserServices is set to true
    ' Parameters:   FormName - The form name where the error occurred
    '               ProcedureName - The procedure name where the error occurred
    '               ExtraDebugInfo - Any additional debuging information
    '***************************************************************************************
    Public Sub FormErrorHandler(ByRef FormName As String, ByRef ProcedureName As String, Optional ByRef ExtraDebugInfo As String = "")
        Dim poErrObject As New frmErrorDetails

        Call LogError(FormName, ProcedureName, Err.GetException, ExtraDebugInfo) 'Log the error
        'Show error form modally
        Call poErrObject.Display(FormName, ProcedureName, Err.GetException, ExtraDebugInfo)
    End Sub
    '***************************************************************************************
    ' Sub:    FormErrorHandler
    ' --------------------------------------------------------------------------------------
    ' Purpose:      Handles untrapped errors generated in form modules
    '               Any errors will be displayed on the errors form
    '               NB: only compiled into the exe when UserServices is set to true
    ' Parameters:   Exception
    '               FormName - The form name where the error occurred
    '               ProcedureName - The procedure name where the error occurred
    '               ExtraDebugInfo - Any additional debuging information
    '***************************************************************************************
    Public Sub FormErrorHandler(ByRef FormName As String, ByRef ProcedureName As String, ByVal ex As Exception, Optional ByRef ExtraDebugInfo As String = "")
        Dim poErrObject As New frmErrorDetails

        Call LogError(FormName, ProcedureName, ex, ExtraDebugInfo) 'Log the error
        'Show error form modally
        Call poErrObject.Display(FormName, ProcedureName, ex, ExtraDebugInfo)
    End Sub
#End If

    '***************************************************************************************
    ' Sub:    LogError
    ' --------------------------------------------------------------------------------------
    ' Purpose:      Creates a standard error message from the err object
    ' Parameters:   ModuleName - Module Name where error was trapped
    '               ProcedureName - Procedure / Function where error was trapped
    '               ex - Exception holding details of error
    '               ExtraDebugInfo - extra debugging information
    '***************************************************************************************
    Private Sub LogError(ByRef ModuleName As String, ByRef ProcedureName As String, ByVal ex As Exception, Optional ByRef ExtraDebugInfo As String = "")
        Dim sErrorString As String

        sErrorString = "Unexpected error occured in " & vbCrLf & _
                       "Application  : " & VB6.GetEXEName & vbCrLf & _
                       "Version      : " & CStr(AppVersion()) & vbCrLf & _
                       "Error Number : " & Err.Number & vbCrLf & _
                       "Description  : " & Err.Description & vbCrLf & _
                       "Module       : " & ModuleName & vbCrLf & _
                       "Procedure    : " & ProcedureName & vbCrLf & _
                       "Line Number  : " & Err.Erl & vbCrLf & _
                       "Extra Info   : " & ExtraDebugInfo & vbCrLf & _
                       "Date         : " & CStr(Now) & vbCrLf & _
                       "UserID       : " & System.Environment.UserName & vbCrLf & _
                       "Calling Stack: " & vbCrLf & ex.StackTrace

        System.Diagnostics.EventLog.WriteEntry(VB6.GetEXEName, sErrorString)
    End Sub

    '***************************************************************************************
    ' Sub:    NoteError
    ' --------------------------------------------------------------------------------------
    ' Purpose:      Appends error details to Err.Source
    ' Parameters:   ModuleName - The module name where the error occurred
    '               ProcedureName - The procedure name where the error occurred
    '               ExtraDebugInfo - Any additional debuging information
    '***************************************************************************************
    Private Sub NoteError(ByRef ModuleName As String, ByRef ProcedureName As String, Optional ByRef ExtraDebugInfo As String = "")


        If Err.Source = VB6.GetEXEName() Then 'Check error source isn't just the project name
            Err.Source = "" 'If it is then clear the error.source
        End If

        Err.Source = Err.Source & vbCrLf & VB6.GetEXEName() & "." & ModuleName & "." & ProcedureName 'Append error details to Err.Source

        'UPGRADE_NOTE: IsMissing() was changed to IsNothing(). Click for more: 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="vbup1021"'
        If Not IsNothing(ExtraDebugInfo) Then
            Err.Source = Err.Source & "." & ExtraDebugInfo 'Append any extra debug info to Err.Source
        End If

    End Sub

    '***************************************************************************************
    ' Sub:    Assert
    ' --------------------------------------------------------------------------------------
    ' Purpose:      To assert that the expression is true
    ' Parameters:   Expression - Expression that must be true
    '               ErrorDescription - Error description
    '***************************************************************************************
    Public Sub Assert(ByRef Expression As Boolean, ByRef ErrorDescription As String)

        If Not (Expression) Then 'Is the expression false
            Err.Number = isgAssertionError 'Set the error number
            Err.Description = ErrorDescription 'Set the error description
            Err.Raise(Err.Number) 'Raise an error
        End If

    End Sub
    '----------------------------------------------
    ' Sub: "AppVersion"
    ' Description: Return application version as single number
    '----------------------------------------------
    Public Function AppVersion() As Single
        AppVersion = System.Diagnostics.FileVersionInfo.GetVersionInfo(System.Reflection.Assembly.GetExecutingAssembly.Location).FileMajorPart + (System.Diagnostics.FileVersionInfo.GetVersionInfo(System.Reflection.Assembly.GetExecutingAssembly.Location).FileMinorPart / 10) + (System.Diagnostics.FileVersionInfo.GetVersionInfo(System.Reflection.Assembly.GetExecutingAssembly.Location).FileBuildPart / 1000)
    End Function
End Module